//sketch for both the receiver and the transmitter
#include <SPI.h>
#include "RF24.h"
#include <Servo.h>

#define throttlePin (A2)
#define aileronPin (A1)
#define elevatorPin (A0)
#define gimbalLeft (2)
#define gimbalUp (3)
#define gimbalRight (4)
#define gimbalDown (5)
#define gimbalReset (6)
#define trigger (9)
#define pinkyTrigger (10)
#define leftThumbButton (A6)
#define switchy (A7)
#define motorPin (3)

Servo elevatorS;
Servo aileronS;
//Servo rudderS;
//Servo cameraXS;
//Servo cameraYS;

RF24 radio(7,8);

struct dataStruct1{
  byte throttle = 0;
  byte aileron = 128;
  byte elevator = 128;
  //byte rudder = 128;
  byte buttonStates = 0;                                          //all buttons: the order of which is explained in a seperate file in the sketch folder
}transmittedData;

bool leftThumbButtonButton, switchyButton;

typedef enum { receiver = 1, transmitter } roleType;              //maye your own datatype
//const roleType role = receiver;                                   //comment this and uncomment the one below this one
const roleType role = transmitter;
const uint64_t pipes[2] = { 0xABCDABCD71LL, 0x544d52687CLL };     //some adresses for the NRF24L01+


void setup() 
{
  if(role == receiver)
  {
    elevatorS.attach(5);
    aileronS.attach(6);
    //rudderS.attach(idk dude);
    //cameraXS.attach(9);
    //cameraYS.attach(10);
    
    //setup the motor
    analogWrite(motorPin, 130);
  }//if(role == receiver)
////////////////////////////////////////////////////////////////////
  if(role == transmitter)
  {
    pinMode(gimbalLeft, INPUT_PULLUP);
    pinMode(gimbalRight, INPUT_PULLUP);
    pinMode(gimbalUp, INPUT_PULLUP);
    pinMode(gimbalDown, INPUT_PULLUP);
    pinMode(gimbalReset, INPUT_PULLUP);
    pinMode(trigger, INPUT_PULLUP);
    pinMode(pinkyTrigger, INPUT_PULLUP);
    //don't forget to put some large resistors between 5v and A6 and A7
  }//if(role == transmitter)
  
  Serial.begin(115200);                                                                                                                                                                     //debugging
  
  radio.begin();                                                  //begin the nrf radio
  radio.setAutoAck(1);                                            //for error correcting code
  radio.setRetries(15,15);                                        //tell the radio to not give up too hastely
  
  if(role == transmitter){                                        //for the role of transmitter open pipes on the adresses one way
    // become the transmitter
    radio.openWritingPipe(pipes[0]);
    radio.openReadingPipe(1,pipes[1]);
  }else{                                                          //for the role of receiver open pipes on the adresses the other way
    // become the receiver
    radio.openWritingPipe(pipes[1]);
    radio.openReadingPipe(1,pipes[0]);
  }
  radio.startListening();                                         //Start listening
  
}
//------------------------------------------------------------------
void loop() 
{
  if(role == receiver)
  {
    //read the data
    if( radio.available()){
      while (radio.available()) {                                 //While there is data ready
        radio.read(&transmittedData, sizeof(transmittedData));    //read the data from the receiver
      }
    }
    Serial.print("  aileron: ");
    Serial.print(transmittedData.aileron);
    Serial.print("  elevator: ");
    Serial.print(transmittedData.elevator);
    Serial.print("  throttle: ");
    Serial.print(transmittedData.throttle);
    Serial.print("  buttons: ");
    Serial.println(transmittedData.buttonStates);                                                                                                                                           //debugging
    
    //write the positions to the servos via the servo.h library
    elevatorS.write(map(transmittedData.elevator, 0, 255, 0, 180));
    aileronS.write(map(transmittedData.aileron, 0, 255, 0, 180));
    //rudderS.write(map(transmittedData.rudder, 0, 255, 0, 180));
    
    //the camera position
    //cameraXS.write(map(transmittedData.aileron, 0, 255, 0, 180));
    //cameraYS.write(map(transmittedData.aileron, 0, 255, 0, 180));
    
    //the motor
    analogWrite(motorPin, map(transmittedData.throttle, 0, 255, 130, 254));
  }//if(role == receiver)
////////////////////////////////////////////////////////////////////
  
  if(role == transmitter)
  {
    updateSensorData();                                           //update the sensor data
    //send the data
    radio.stopListening();                                        //stop listening to send something
    if ( !radio.write(&transmittedData,sizeof(transmittedData)) )             // Send the button state(s) to the server and at the same time check if it's succesfull
    {
      //put the code that you want to run on an unsuccesfull sending attempt
      Serial.println("sending faillure");                                                                                                                                                   //debugging
    }
    
    
  }//if(role == transmitter)

  Serial.println();
  delay(1);
}
//-------------------------------------------------------------------

//the script for updating the sensor data (only for the transmitter);
void updateSensorData()
{
  //read the throttle, elevator and aileron axis
  transmittedData.throttle = map(analogRead(throttlePin), 0, 1023, 255, 0);
  transmittedData.aileron = map(analogRead(aileronPin), 0, 1023, 0, 255);
  transmittedData.elevator = map(analogRead(elevatorPin), 0, 1023, 0, 255);
  //read the buttons and switch and code them into one byte
  transmittedData.buttonStates = 0;
  transmittedData.buttonStates = transmittedData.buttonStates | (!digitalRead(gimbalReset) << 4);
  transmittedData.buttonStates = transmittedData.buttonStates | (!digitalRead(trigger) << 3);
  transmittedData.buttonStates = transmittedData.buttonStates | (!digitalRead(pinkyTrigger) << 2);
  if(analogRead(leftThumbButton) == 0) { leftThumbButtonButton = 0; } else{ leftThumbButtonButton = 1; }
  transmittedData.buttonStates = transmittedData.buttonStates | ((!leftThumbButtonButton) << 1);
  if(analogRead(switchy) == 0) { switchyButton = 0; } else{ switchyButton = 1; }
  transmittedData.buttonStates = transmittedData.buttonStates | ((!switchyButton) << 0);
  
  if(!digitalRead(gimbalLeft)){
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 7);  //code for gimbalLeft
    transmittedData.buttonStates = transmittedData.buttonStates | ( 0 << 6);
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 5);  //activation bit
  }
  if(!digitalRead(gimbalRight)){
    transmittedData.buttonStates = transmittedData.buttonStates | ( 0 << 7);  //code for gimbalRight
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 6);
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 5);  //activation bit
  }
  if(!digitalRead(gimbalUp)){
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 7);  //code for gimbalUp
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 6);
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 5);  //activation bit
  }
  if(!digitalRead(gimbalDown)){
    transmittedData.buttonStates = transmittedData.buttonStates | ( 0 << 7);  //code for gimbalDown
    transmittedData.buttonStates = transmittedData.buttonStates | ( 0 << 6);
    transmittedData.buttonStates = transmittedData.buttonStates | ( 1 << 5);  //activation bit
  }
  if((digitalRead(gimbalLeft)) && (digitalRead(gimbalRight)) &&  (digitalRead(gimbalUp)) &&  (digitalRead(gimbalDown)))
  {
    transmittedData.buttonStates = transmittedData.buttonStates | ( 0 << 5);  //deactivation bit (for 
  }
  
  Serial.println(transmittedData.buttonStates);                                                                                                                                              //debugging
  
}

